#!/usr/bin/env python

### BEGIN INIT INFO
# Provides:          tinker-access-client
# Required-Start:    $remote_fs $syslog $network
# Required-Stop:     $remote_fs $syslog
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: tinker-access-client
# Description:       The tinker-access-client service is responsible for coordinating communication
# between RPi's modules (i.e. RFID reader, LCD, power relays etc..) and the remote tinker_access_server.
### END INIT INFO

import sys
import logging
from ClientDaemon import ClientDaemon
from ClientOption import ClientOption
from CommandHandler import CommandHandler
from ClientOptionParser import ClientOptionParser, Command


def run():
    (opts, args) = ClientOptionParser().parse_args()

    try:

        # TODO: whats the best place to handle this configuration of the logger?
        sys_log_format = \
            '%(asctime)s %(hostname)s %(app_id)s %(version)s %(device_id)s %(device_name)s ' \
            '%(user_id)s %(user_name)s %(badge_code)s %(message)s'

        logging.basicConfig(
            datefmt='%FT%T',
            level=logging.DEBUG,
            format='%(asctime)s %(message)s',
            filename=opts.get(ClientOption.LOG_FILE)
        )

        with CommandHandler(opts, args) as handler:
            handler.on(Command.STOP, ClientDaemon.stop)
            handler.on(Command.START, ClientDaemon.start)
            handler.on(Command.STATUS, ClientDaemon.status)
            handler.on(Command.RESTART, ClientDaemon.restart)
            return handler.handle_command()

    except Exception as e:
        sys.stdout.write(str(e))
        sys.stdout.flush()
        sys.exit(1)

    finally:
        logging.shutdown()

if __name__ == '__main__':
    run()



import os
import tempfile
import subprocess

from PackageInfo import PackageInfo
from ClientLogger import ClientLogger


class ServiceInstaller(object):
    def __init__(self, install_lib):
        self.__logger = ClientLogger.setup(phase='install')

        self.__install_lib = install_lib
        self.__service_link = "/etc/init.d/{0}".format(PackageInfo.pip_package_name)
        self.__service_script = '{0}{1}/Service.py'.format(install_lib, PackageInfo.python_package_name)

    def install(self):
        try:
            self.__remove_legacy_client()  # TODO: remove after upgrade
            self.__create_service()
            self.__configure_service()

            # TODO: configure/enable auto-updates?

        except Exception as e:
            self.__logger.debug('%s service installation failed.', PackageInfo.pip_package_name)
            self.__logger.exception(e)
            raise e

    # noinspection PyMethodMayBeStatic
    def __ensure_execute_permission(self, path):
        os.chmod(path, 0755)

    # TODO: this can be removed after the upgrade
    def __remove_legacy_client(self):
        self.__execute_commands([
            'service tinkerclient stop\n',
            'update-rc.d -f tinkerclient remove\n',
            'rm -rf /etc/init.d/tinkerclient\n',
            'rm -rf /opt/tinkeraccess/client.py\n',
            'rm -rf /opt/tinkeraccess/client.pyc\n',
            'rm -rf /opt/tinkeraccess/lcdModule.py\n',
            'rm -rf /opt/tinkeraccess/lcdModule.pyc\n',
            'rm -rf /opt/tinkeraccess/client.cfg\n'
        ])

    def __create_service(self):
        self.__ensure_execute_permission(self.__service_script)

        # remove any existing service if it is a file or directory, and it is not a symlink
        if os.path.exists(self.__service_link) and not os.path.islink(self.__service_link):
            os.remove(self.__service_link)

        # remove the existing service if it is a symlink and it is not pointed to the current target
        if os.path.lexists(self.__service_link) and os.readlink(self.__service_link) != self.__service_script:
            os.remove(self.__service_link)

        # create the symlink if it doesn't already exists
        if not os.path.lexists(self.__service_link):
            os.symlink(self.__service_script, self.__service_link)

    def __configure_service(self):
        self.__execute_commands([
            'update-rc.d {0} defaults 91\n'.format(PackageInfo.pip_package_name),
            'service {0} restart\n'.format(PackageInfo.pip_package_name)
        ])

    def __execute_commands(self, commands):
        # I suppose an explanation is warranted here...
        # Unfortunately we cannot execute these commands directly from python due to the fact that the
        # start priority 91 must be passed to the update-rc command as an integer and python converts all arguments to
        # strings which causes and exception when the update command is invoked.
        # We work around the problem by creating a temporary script file and executing that
        fd, path = tempfile.mkstemp(suffix='.sh')
        try:
            with os.fdopen(fd, 'w') as tmp:
                tmp.writelines(['#!/usr/bin/env bash\n\n'] + commands)
            self.__execute_script(path)
        finally:
            os.remove(path)

    def __execute_script(self, path):
        self.__ensure_execute_permission(path)

        cmd = [path] + ['-evx']  # Options: http://www.tldp.org/LDP/abs/html/options.html
        cmd_process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout_data, stderr_data = cmd_process.communicate()
        if cmd_process.returncode != 0:
            for ln in stderr_data.splitlines(True):
                self.__logger.error(ln)
            raise RuntimeError('{0} command failed.'.format(cmd))





import os
import sys
import signal
import time
import logging
from subprocess import \
    check_output, \
    CalledProcessError

from Client import Client
from Command import Command
from daemonize import Daemonize
from PackageInfo import PackageInfo
from ClientOption import ClientOption


# noinspection PyClassHasNoInit
class ClientDaemon:

    @staticmethod
    def start(opts, args):
        logger = logging.getLogger(__name__)
        if not ClientDaemon.__status(opts, args):
            try:
                pid_file = opts.get(ClientOption.PID_FILE)
                foreground = opts.get(ClientOption.DEBUG)

                def start():
                    Client.run(opts, args)

                daemon = Daemonize(
                    app=PackageInfo.pip_package_name,
                    pid=pid_file,
                    action=start,
                    foreground=foreground,
                    verbose=True,
                    logger=logger,
                    auto_close_fds=False
                )
                daemon.start()
            except Exception as e:
                logger.debug('%s start failed.', PackageInfo.pip_package_name)
                logger.exception(e)
                raise e
        else:
            sys.stdout.write('{0} is already running...\n'.format(PackageInfo.pip_package_name))
            sys.stdout.flush()
            sys.exit(1)

    @staticmethod
    def stop(opts, args):
        logger = logging.getLogger(__name__)
        pid_file = opts.get(ClientOption.PID_FILE)
        logout_coast_time = opts.get(ClientOption.LOGOUT_COAST_TIME)
        max_power_down_timeout = opts.get(ClientOption.MAX_POWER_DOWN_TIMEOUT)

        try:
            for pid in check_output(['pgrep', '-f', '(/{0}\s+(start|restart))'.format(PackageInfo.pip_package_name)]).splitlines():
                is_current_pid = int(pid) == int(os.getpid())
                if not is_current_pid:
                    try:
                        os.kill(int(pid), signal.SIGTERM)
                    except Exception as e:
                        logger.exception(e)
        except CalledProcessError:
            pass

        # Wait for what we assume will be a graceful exit,
        # this will exit early if the pid file is removed as expected.
        current = time.time()
        max_wait_time = max_power_down_timeout + logout_coast_time + 5
        while time.time() - current < max_wait_time and os.path.isfile(pid_file):
            time.sleep(0.1)

        # This should never happen, but... if the pid file still exist at this point, we will nuke it from orbit!
        # Otherwise it will prevent the client daemon from starting again...
        if os.path.isfile(pid_file):
            try:
                os.remove(pid_file)
            except Exception as e:
                logger.exception(e)

    @staticmethod
    def restart(opts, args):
        logger = logging.getLogger(__name__)
        restart_delay = opts.get(ClientOption.RESTART_DELAY)
        try:
            args[0] = Command.STOP.get('command')
            ClientDaemon.stop(opts, args)
            logger.debug('Restarting in %s seconds...', restart_delay)
            time.sleep(restart_delay)
            args[0] = Command.START.get('command')
            ClientDaemon.start(opts, args)
        except Exception as e:
            logger.debug('%s restart failed.', PackageInfo.pip_package_name)
            logger.exception(e)
            raise e

    @staticmethod
    def status(opts, args):
        status = ClientDaemon.__status(opts, args)
        if status:
            sys.stdout.write(status)
            sys.stdout.flush()
            sys.exit(0)
        else:
            sys.stdout.write('terminated\n')
            sys.stdout.flush()
            sys.exit(1)

    @staticmethod
    def __status(opts, _):
        status_file = opts.get(ClientOption.STATUS_FILE)

        status = terminated = 'terminated'
        if os.path.isfile(status_file):
            with open(status_file, 'r') as f:
                status = f.readline()

        return status if status is not terminated else None




import os
import logging
import logging.handlers as handlers
from PackageInfo import PackageInfo
from ContextFilter import ContextFilter
from ClientOptionParser import ClientOptionParser, ClientOption


# TODO: should this all just be done with a config file?

class ClientLogger(object):
    @staticmethod
    def setup(phase=None):
        logger = logging.getLogger(PackageInfo.python_package_name)
        (opts, args) = ClientOptionParser(phase=phase).parse_args()

        logger.filters = []
        logger.handlers = []

        debug_mode = opts.get(ClientOption.DEBUG)
        log_level = opts.get(ClientOption.LOG_LEVEL)
        if debug_mode:
            logger.setLevel(10)
        else:
            logger.setLevel(log_level if log_level else 40)

        logger.addFilter(ContextFilter(opts))

        sys_log_format = \
            '%(asctime)s %(hostname)s %(app_id)s %(version)s %(device_id)s %(device_name)s ' \
            '%(user_id)s %(user_name)s %(badge_code)s %(message)s'
        sys_log_formatter = logging.Formatter(sys_log_format, datefmt='%FT%T')

        # dev/log
        sys_log_file = '/var/log/syslog'
        if os.path.exists(sys_log_file):
            sys_log_handler = handlers.SysLogHandler('/dev/log')
            logger.addHandler(sys_log_handler)
            sys_log_handler.setFormatter(sys_log_formatter)

        # console
        logger.addHandler(logging.StreamHandler())

        # fileHandler -
        try:
            log_file = opts.get(ClientOption.LOG_FILE)
            if log_file:
                if not os.path.exists(os.path.dirname(log_file)):
                    os.makedirs(os.path.dirname(log_file))

                file_handler = handlers.TimedRotatingFileHandler(log_file, when='D', interval=1, backupCount=7)
                logger.addHandler(file_handler)
        except Exception:
            pass

        return logger
